unit package HomoGlypher::Mappings;

=begin pod

Rules:

Keys should contain only ASCII characters.
For example we don't provide mapping from 'ż' to 'ź'.

Keys should be provided in "digts, lowercase letters, uppercase letters" order.

Values should be sorted in "from most to less similar" order.
Glyphs with accents are always considered less similar.

Size difference must be reasonable.
For example 'O' and '°' are not considered homoglyphs.

Cursive similarities are not included.
For example 'т' in cursive looks like 'm'.
Maybe such advanced mappings will be added in the future.

Vertical mirroring are not included in mappings.
For example 'R' and 'Я' does not qualify.

90°, 180° and 270° rotations with optional horizontal or vertical mirroring
are included only in special %flipped mapping.
For example 'G' and '⅁' or 'J' and '𐐓'.

Remember that mappings are font dependent and subjective.
There is no ultimate truth.

=end pod

our %armenian = (

    '2' => [ 'շ' ],
    '8' => [ 'Ց' ],

    'b' => [ 'Ե' ],
    'd' => [ 'Ժ', 'ժ' ],
    'g' => [ 'ց' ],
    'n' => [ 'ո', 'ռ' ],
    'o' => [ 'օ' ],
    'u' => [ 'ս' ],
    'w' => [ 'ա' ],

    'B' => [ 'Յ' ],
    'O' => [ 'Օ', 'Ծ' ],
    'S' => [ 'Տ' ],
    'U' => [ 'Ա', 'Մ', 'Ս' ],

);
    
our %cherokee = (

    '0' => [ 'Ꮻ', 'Ꮎ' ],
    '4' => [ 'Ꮞ' ],
    '6' => [ 'Ꮾ' ],
    '9' => [ 'Ꮽ' ],
    
    'b' => [ 'Ꮟ', 'Ꮾ' ],
    'd' => [ 'Ꮷ', 'Ꮄ' ],
    'f' => [ 'Ꮅ' ],
    'fi' => [ 'Ᏺ' ],
    'h' => [ 'Ꮒ', 'Ꮵ', 'Ᏺ' ],
    'i' => [ 'Ꭵ' ],
    'l' => [ 'Ꮣ' ],
    'r' => [ 'Ꮁ' ],
    'w' => [ 'Ꮗ', 'Ꮚ' ],
    'y' => [ 'Ꭹ', 'Ꮍ' ],

    'A' => [ 'Ꭺ', 'Ꭿ', 'Ꮑ' ],
    'B' => [ 'Ᏼ', 'Ᏸ' ],
    'C' => [ 'Ꮯ', 'Ꮳ', 'Ꮸ' ],
    'D' => [ 'Ꭰ' ],
    'E' => [ 'Ꭼ', 'Ꮛ' ],
    'G' => [ 'Ꮐ', 'Ꮹ', 'Ꮆ', 'Ᏻ' ],
    'H' => [ 'Ꮋ' ],
    'I' => [ 'Ꭸ', 'Ꮀ', 'Ꮠ' ],
    'J' => [ 'Ꭻ' ],
    'K' => [ 'Ꮶ' ],
    'L' => [ 'Ꮮ', 'Ꮭ' ],
    'M' => [ 'Ꮇ' ],
    'O' => [ 'Ꮎ', 'Ꮻ', 'Ꭴ', 'Ꮕ' ],
    'OI' => [ 'Ꮊ' ],
    'P' => [ 'Ꮲ', 'Ꭾ' ],
    'R' => [ 'Ꭱ', 'Ꮢ' ],
    'S' => [ 'Ꮪ', 'Ꮥ', 'Ꭶ' ],
    'T' => [ 'Ꭲ' ],
    'U' => [ 'Ꮜ' ],
    'V' => [ 'Ꮩ', 'Ꮴ', 'Ꮙ' ],
    'W' => [ 'Ꮃ', 'Ꮤ' ],
    'Z' => [ 'Ꮓ' ],

);

our %deseret = (

    '6' => [ '𐐞' ],

    'a' => [ '𐐀', '𐐀' ],
    'b' => [ '𐑇' ],
    'c' => [ '𐐽' ],
    'd' => [ '𐐼' ],
    'l' => [ '𐑊', '𐐢' ],
    'o' => [ '𐐬', '𐐫' ],
    'p' => [ '𐑁' ],
    's' => [ '𐑈' ],
    'v' => [ '𐐷' ],
    'w' => [ '𐐶' ],
    'y' => [ '𐑄' ],

    'B' => [ '𐐚' ],
    'C' => [ '𐐕' ],
    'D' => [ '𐐟' ],
    'E' => [ '𐐁' ],
    'J' => [ '𐐢' ],
    'L' => [ '𐐛', '𐑃' ],
    'O' => [ '𐐄', '𐐃' ],
    'P' => [ '𐐙' ],
    'S' => [ '𐐠' ],
    'V' => [ '𐐏' ],
    'W' => [ '𐐎' ],
    'Y' => [ '𐐜' ],

);

our %cyrillic = (

    '0' => [ 'О', 'Ф', 'ф' ],
    '3' => [ 'З', 'з' ],

    'a' => [ 'а' ],
    'b' => [ 'Ь', 'ь', 'б', 'Б', ],
    'c' => [ 'с' ],
    'e' => [ 'е' ],
    'i' => [ 'і' ],
    'io' => [ 'ю' ],
    'n' => [ 'п' ],
    'o' => [ 'о' ],
    'p' => [ 'р' ],
    'r' => [ 'г', 'Г' ],
    's' => [ 'ѕ' ],
    'u' => [ 'ц' ],
    'v' => [ 'ѵ' ],
    'w' => [ 'ш', 'щ', 'ѡ' ],
    'x' => [ 'х' ],
    'y' => [ 'у', 'У', 'ч', 'Ч' ],

    'A' => [ 'А' ],
    'B' => [ 'В', 'в' ],
    'C' => [ 'С' ],
    'E' => [ 'Е' ],
    'H' => [ 'Н', 'н' ],
    'I' => [ 'І' ],
    'IO' => [ 'Ю' ],
    'K' => [ 'К', 'к' ],
    'M' => [ 'М', 'м' ],
    'O' => [ 'О' ],
    'P' => [ 'Р' ],
    'S' => [ 'Ѕ' ],
    'T' => [ 'Т', 'т' ],
    'U' => [ 'Ц' ],
    'V' => [ 'Ѵ' ],
    'W' => [ 'Ш', 'Щ', 'Ѡ' ],
    'X' => [ 'Х' ],
    'Y' => [ 'У', 'Ч' ],

);

our %greek = (

    '0' => [ 'Θ', 'θ' ],

    'a' => [ 'α' ],
    'c' => [ 'ϲ', 'ς' ],
    'l' => [ 'ι' ],
    'n' => [ 'η', 'π' ],
    'o' => [ 'ο', 'σ' ],
    'p' => [ 'ρ' ],
    'r' => [ 'Γ' ],
    'u' => [ 'υ', 'μ' ],
    'v' => [ 'ν' ],
    'w' => [ 'ω', 'ψ' ],
    'y' => [ 'γ' ],

    'A' => [ 'Α', '𝚲', 'Δ' ],
    'B' => [ 'Β', 'β' ],
    'E' => [ 'Ε', 'Σ', 'ε', 'ξ' ],
    'H' => [ 'Η' ],
    'I' => [ 'Ι' ],
    'K' => [ 'Κ', 'κ' ],
    'M' => [ 'Μ' ],
    'N' => [ 'Ν' ],
    'O' => [ 'Ο', 'Θ', 'θ', 'Ω' ],
    'P' => [ 'Ρ' ],
    'T' => [ 'Τ', 'τ' ],
    'W' => [ 'Ψ' ],
    'X' => [ 'Χ', 'χ' ],
    'Y' => [ 'Υ', 'γ' ],
    'Z' => [ 'Ζ' ],

);

our %greek-mathematical-typeface = (

    '0' => [ '𝛉', '𝜃', '𝜽', '𝝷', '𝞱', '𝚯', '𝚹', '𝛩', '𝛳', '𝜣', '𝜭', '𝝝', '𝝧', '𝞗', '𝞡', '𝚯', '𝚹', '𝛩', '𝛳', '𝜣', '𝜭', '𝝝', '𝝧', '𝞗', '𝞡' ],

    'a' => [ '𝛂', '𝛼', '𝜶', '𝝰', '𝞪' ],
    'c' => [ '𝞁', '𝞻' ],
    'l' => [ '𝛊', '𝜄', '𝜾', '𝝸', '𝞲' ],
    'n' => [ '𝛈', '𝜂', '𝜼', '𝝶', '𝞰' ],
    'o' => [ '𝛐', '𝜊', '𝝄', '𝝾', '𝞸', '𝛔', '𝜎', '𝞂', '𝞼' ],
    'p' => [ '𝛒', '𝞀', '𝞺' ],
    'q' => [ '𝜌', '𝝆', ],
    'r' => [ '𝚪', '𝛤', '𝜞', '𝝘', '𝞒' ],
    't' => [ '𝛕', '𝜏', '𝝉', '𝞃', '𝞽' ],
    'u' => [ '𝛍', '𝜇', '𝝁', '𝝻', '𝞵', '𝛖', '𝜐', '𝝊', '𝞄', '𝞾' ],
    'v' => [ '𝛎', '𝜈', '𝝂', '𝝼', '𝞶' ],
    'w' => [ '𝛚', '𝜔', '𝝎', '𝞈', '𝟂', '𝛙', '𝜓', '𝝍', '𝞇', '𝟁' ],
    'y' => [ '𝛄', '𝛾', '𝜸', '𝝲', '𝞬' ],

    'A' => [ '𝚨', '𝛢', '𝜜', '𝝖', '𝞐', '𝚲', '𝛬', '𝜦', '𝝠', '𝞚', '𝚫', '𝛥', '𝜟', '𝝙', '𝞓' ],
    'B' => [ '𝚩', '𝛣', '𝜝', '𝝗', '𝞑', '𝛃', '𝛽', '𝜷', '𝝱', '𝞫' ],
    'E' => [ '𝚬', '𝛦', '𝜠', '𝝚', '𝞔', '𝚺', '𝛴', '𝜮', '𝝨', '𝞢', '𝛆', '𝜀', '𝜺', '𝝴', '𝞮', '𝛏', '𝜉', '𝝃', '𝝽', '𝞷' ],
    'F' => [ '𝟊', '𝟋' ],
    'H' => [ '𝚮', '𝛨', '𝜢', '𝝜', '𝞖' ],
    'I' => [ '𝚰', '𝛪', '𝜤', '𝝞', '𝞘' ],
    'K' => [ '𝚱', '𝛫', '𝜥', '𝝟', '𝞙', '𝛋', '𝜅', '𝜿', '𝝹', '𝞳' ],
    'M' => [ '𝚳', '𝛭', '𝜧', '𝝡', '𝞛' ],
    'N' => [ '𝚴', '𝛮', '𝜨', '𝝢', '𝞜' ],
    'O' => [ '𝚶', '𝛰', '𝜪', '𝝤', '𝞞', '𝛉', '𝜃', '𝜽', '𝝷', '𝞱', '𝚯', '𝚹', '𝛩', '𝛳', '𝜣', '𝜭', '𝝝', '𝝧', '𝞗', '𝞡', '𝚯', '𝚹', '𝛩', '𝛳', '𝜣', '𝜭', '𝝝', '𝝧', '𝞗', '𝞡', '𝛀', '𝛺', '𝜴', '𝝮', '𝞨' ],
    'P' => [ '𝚸', '𝛲', '𝜬', '𝝦', '𝞠' ],
    'T' => [ '𝚻', '𝛵', '𝜯', '𝝩', '𝞣' ],
    'W' => [ '𝚿', '𝛹', '𝜳', '𝝭', '𝞧' ],
    'X' => [ '𝚾', '𝛸', '𝜲', '𝝬', '𝞦', '𝛘', '𝜒', '𝝌', '𝞆', '𝟀' ],
    'Y' => [ '𝚼', '𝛶', '𝜰', '𝝪', '𝞤', '𝛄', '𝛾', '𝜸', '𝝲', '𝞬' ],
    'Z' => [ '𝚭', '𝛧', '𝜡', '𝝛', '𝞕' ],

);

our %georgian = (

    '3' => [ 'ვ', 'კ', 'პ', 'ჳ', 'ჴ' ],

    'b' => [ 'Ⴆ', 'ხ', 'Ⴊ', 'ს', ],
    'd' => [ 'ძ', 'ქ' ],
    'h' => [ 'Ⴙ' ],
    'l' => [ 'Ⴑ', 'Ⴠ' ],
    'm' => [ 'ო' ],
    'o' => [ 'ი' ],
    'y' => [ 'Ⴘ', 'Ⴁ', 'Ⴏ', 'Ⴣ', 'Ⴗ', 'ყ' ],

    'C' => [ 'Ⴚ' ],
    'J' => [ 'Ⴐ' ],
    'O' => [ 'Ⴍ', 'Ⴍ', 'Ⴓ' ],
    'S' => [ 'Ⴝ' ],
    'U' => [ 'Ⴎ' ],
    'Y' => [ 'Ⴤ' ],

);

our %latin = (

    '0' => [ 'ʘ' ],
    '3' => [ 'Ʒ', 'Ȝ', 'ȝ', 'ɜ', 'ʒ', 'ᴈ', 'ᴣ', 'Ꝫ', 'ꝫ',  ],
    '5' => [ 'Ƽ', 'ƽ' ],
    '6' => [ 'Ỽ' ],
    '8' => [ 'ꝸ' ],
    '9' => [ 'Ꝯ' ],

    'aa' => [ 'ꜳ' ],
    'ae' => [ 'æ' ],
    'ao' => [ 'ꜵ' ],
    'au' => [ 'ꜷ' ],
    'av' => [ 'ꜹ' ],
    'b' => [ 'Ƅ', 'ƅ' ],
    'd' => [ 'ꝱ' ],
    'dz' => [ 'ǳ', 'ʣ' ],
    'f' => [ 'ſ', 'ẜ', 'ẝ' ],
    'ff' => [ 'ﬀ' ],
    'ffi' => [ 'ﬃ' ],
    'ffl' => [ 'ﬄ' ],
    'fi' => [ 'ﬁ' ],
    'fl' => [ 'ﬂ' ],
    'fn' => [ 'ʩ' ],
    'g' => [ 'ɡ', 'ꝯ' ],
    'h' => [ 'ꜧ' ],
    'hv' => [ 'ƕ' ],
    'ij' => [ 'ĳ' ],
    'j' => [ 'ȷ' ],
    'l' => [ 'ɩ', 'Ɩ', 'ꝲ' ],
    'lj' => [ 'ǈ', 'ǉ' ],
    'ls' => [ 'ʪ' ],
    'lz' => [ 'ʫ' ],
    'm' => [ 'ꝳ' ],
    'n' => [ 'ꝴ' ],
    'nj' => [ 'ǌ' ],
    'o' => [ 'ᴏ' ],
    'oe' => [ 'œ' ],
    'oi' => [ 'ƣ' ],
    'oo' => [ 'ꝏ' ],
    'p' => [ 'ƿ', 'þ' ],
    'r' => [ 'ꝵ', 'ꞅ' ],
    'st' => [ 'ﬆ' ],
    't' => [ '✝' ],
    'tf' => [ 'ʧ' ],
    'ts' => [ 'ʦ' ],
    'u' => [ 'ᴜ' ],
    'v' => [ 'ᴠ' ],
    'vy' => [ 'ꝡ' ],
    'w' => [ 'ᴡ' ],
    'z' => [ 'ᴢ' ],

    'A' => [ 'ᴀ' ],
    'AA' => [ 'Ꜳ' ],
    'AE' => [ 'Æ', 'ᴁ' ],
    'AJ' => [ 'Ꜷ' ],
    'AO' => [ 'Ꜵ' ],
    'AV' => [ 'Ꜹ' ],
    'AY' => [ 'Ꜽ' ],
    'B' => [ 'ʙ', 'ß' ],
    'C' => [ 'ᴄ', 'ʗ' ],
    'D' => [ 'ᴅ', 'Ð', 'Ɖ' ],
    'Dz' => [ 'ǲ' ],
    'DZ' => [ 'Ǳ' ],
    'E' => [ 'ᴇ', 'Ɛ', 'Ʃ', 'ɛ', 'Ꜫ', 'ꜫ' ],
    'F' => [ 'ꜰ', 'ꝼ' ],
    'G' => [ 'ɢ' ],
    'H' => [ 'ʜ', 'Ꜧ' ],
    'Hi' => [ 'Ƕ' ],
    'I' => [ 'ǀ', 'ɪ' ],
    'II' => [ 'ǁ' ],
    'IJ' => [ 'Ĳ' ],
    'J' => [ 'ᴊ' ],
    'K' => [ 'ᴋ', 'ĸ' ],
    'L' => [ 'ʟ' ],
    'LJ' => [ 'Ǉ' ],
    'M' => [ 'ᴍ' ],
    'N' => [ 'ɴ' ],
    'Nj' => [ 'ǋ', 'Ǌ' ],
    'O' => [ 'ᴏ', 'Ꝺ' ],
    'OE' => [ 'Œ' ],
    'OI' => [ 'Ƣ' ],
    'OO' => [ 'Ꝏ' ],
    'P' => [ 'ᴘ', 'Ƿ' ],
    'R' => [ 'ʀ', 'Ʀ', 'ꝶ' ],
    'S' => [ 'ꜱ' ],
    'T' => [ 'ᴛ' ],
    'U' => [ 'ᴜ', 'Ʊ' ],
    'V' => [ 'ᴠ' ],
    'VY' => [ 'Ꝡ' ],
    'W' => [ 'ᴡ', 'ꟿ' ],
    'Y' => [ 'ʏ', 'Ɣ' ],
    'Z' => [ 'ᴢ' ],

);

our %letterlike = (

);

our %lisu = (

    'd' => [ 'ꓒ' ],

    'A' => [ 'ꓮ' ],
    'B' => [ 'ꓐ' ],
    'C' => [ 'ꓚ' ],
    'D' => [ 'ꓓ' ],
    'E' => [ 'ꓰ' ],
    'F' => [ 'ꓝ' ],
    'G' => [ 'ꓖ' ],
    'H' => [ 'ꓧ' ],
    'I' => [ 'ꓲ' ],
    'J' => [ 'ꓙ' ],
    'K' => [ 'ꓗ' ],
    'L' => [ 'ꓡ' ],
    'M' => [ 'ꓟ' ],
    'N' => [ 'ꓠ' ],
    'O' => [ 'ꓳ' ],
    'P' => [ 'ꓑ' ],
    'R' => [ 'ꓣ' ],
    'S' => [ 'ꓢ' ],
    'T' => [ 'ꓔ' ],
    'U' => [ 'ꓴ' ],
    'V' => [ 'ꓦ' ],
    'W' => [ 'ꓪ' ],
    'X' => [ 'ꓫ' ],
    'Y' => [ 'ꓬ' ],
    'Z' => [ 'ꓜ' ],

);

our %myanmar = (

    '3' => [ 'ဒ' ],

    'c' => [ 'င' ],
    'm' => [ 'က' ],
    'o' => [ 'ဝ', ],
    'oo' => [ 'ထ' ],
    'w' => [ 'ယ' ],

);

our %roman-numerals = (

    'c' => [ 'ⅽ' ],
    'd' => [ 'ⅾ' ],
    'i' => [ 'ⅰ' ],
    'ii' => [ 'ⅱ' ],
    'iii' => [ 'ⅲ' ],
    'iv' => [ 'ⅳ' ],
    'ix' => [ 'ⅸ' ],
    'l' => [ 'ⅼ' ],
    'm' => [ 'ⅿ' ],
    'v' => [ 'ⅴ' ],
    'vi' => [ 'ⅵ' ],
    'vii' => [ 'ⅶ' ],
    'viii' => [ 'ⅷ' ],
    'x' => [ 'ⅹ' ],
    'xi' => [ 'ⅺ' ],
    'xii' => [ 'ⅻ' ],

    'C' => [ 'Ⅽ', 'ↅ' ],
    'D' => [ 'Ⅾ' ],
    'I' => [ 'Ⅰ' ],
    'II' => [ 'Ⅱ' ],
    'III' => [ 'Ⅲ' ],
    'IV' => [ 'Ⅳ' ],
    'IX' => [ 'Ⅸ' ],
    'L' => [ 'Ⅼ' ],
    'M' => [ 'Ⅿ' ],
    'V' => [ 'Ⅴ' ],
    'VI' => [ 'Ⅵ' ],
    'VII' => [ 'Ⅶ' ],
    'VIII' => [ 'Ⅷ' ],
    'X' => [ 'Ⅹ' ],
    'XI' => [ 'Ⅺ' ],
    'XII' => [ 'Ⅻ' ],

);

our %runic = (

    '1' => [ 'ᛐ' ],

    'B' => [ 'ᛒ' ],
    'E' => [ 'ᛊ' ],
    'I' => [ 'ᛁ' ],
    'K' => [ 'ᛕ' ],
    'M' => [ 'ᛖ' ],
    'N' => [ 'ᚺ' ],
    'P' => [ 'ᚹ' ],
    'R' => [ 'ᚱ' ],
    'X' => [ 'ᚷ' ],

);

our %math-symbols = (

    '0' => [ '∅' ],
    '3' => [ '϶' ],

    'a' => [ '∂' ],
    'c' => [ '⊂' ],
    'f' => [ '⨍' ],
    'n' => [ '⨅' ],
    'u' => [ '∪', '⋃' ],
    'v' => [ '⋁', '⋎' ],
    'x' => [ '⨉', '×' ],

    'C' => [ '∁' ],
    'D' => [ '⫐' ],
    'E' => [ '⋿', '∈', '∑', '⨊' ],
    'I' => [ '|', '⎮', '⎪' ],
    'L' => [ '∟' ],
    'T' => [ '⟙', '⊤', '⥾' ],
    'V' => [ '√' ],

);

# set of all typical homoglyph mappings for easier loading
our @basic = (
    %armenian,
    %cherokee,
    %cyrillic,
    %deseret,
    %greek,
    %greek-mathematical-typeface,
    %georgian,
    %latin,
    %lisu,
    %myanmar,
    %roman-numerals,
    %runic,
    %math-symbols
);

our %ballot = (

    'X' => [ '✗' ],

);

our %ballot-bold-script = (

    'x' => [ '🗶' ],

);

our %ballot-script = (

    'x' => [ '🗴' ],

);

our %bold = (

    '0' => [ '𝟎' ],
    '1' => [ '𝟏' ],
    '2' => [ '𝟐' ],
    '3' => [ '𝟑' ],
    '4' => [ '𝟒' ],
    '5' => [ '𝟓' ],
    '6' => [ '𝟔' ],
    '7' => [ '𝟕' ],
    '8' => [ '𝟖' ],
    '9' => [ '𝟗' ],

    'a' => [ '𝐚' ],
    'b' => [ '𝐛' ],
    'c' => [ '𝐜' ],
    'd' => [ '𝐝' ],
    'e' => [ '𝐞' ],
    'f' => [ '𝐟' ],
    'g' => [ '𝐠' ],
    'h' => [ '𝐡' ],
    'i' => [ '𝐢' ],
    'j' => [ '𝐣' ],
    'k' => [ '𝐤' ],
    'l' => [ '𝐥' ],
    'm' => [ '𝐦' ],
    'n' => [ '𝐧' ],
    'o' => [ '𝐨' ],
    'p' => [ '𝐩' ],
    'q' => [ '𝐪' ],
    'r' => [ '𝐫' ],
    's' => [ '𝐬' ],
    't' => [ '𝐭' ],
    'u' => [ '𝐮' ],
    'v' => [ '𝐯' ],
    'w' => [ '𝐰' ],
    'x' => [ '𝐱' ],
    'y' => [ '𝐲' ],
    'z' => [ '𝐳' ],

    'A' => [ '𝐀' ],
    'B' => [ '𝐁' ],
    'C' => [ '𝐂' ],
    'D' => [ '𝐃' ],
    'E' => [ '𝐄' ],
    'F' => [ '𝐅' ],
    'G' => [ '𝐆' ],
    'H' => [ '𝐇' ],
    'I' => [ '𝐈' ],
    'J' => [ '𝐉' ],
    'K' => [ '𝐊' ],
    'L' => [ '𝐋' ],
    'M' => [ '𝐌' ],
    'N' => [ '𝐍' ],
    'O' => [ '𝐎' ],
    'P' => [ '𝐏' ],
    'Q' => [ '𝐐' ],
    'R' => [ '𝐑' ],
    'S' => [ '𝐒' ],
    'T' => [ '𝐓' ],
    'U' => [ '𝐔' ],
    'V' => [ '𝐕' ],
    'W' => [ '𝐖' ],
    'X' => [ '𝐗' ],
    'Y' => [ '𝐘' ],
    'Z' => [ '𝐙' ],

);

our %bold-fraktur = (

    'a' => [ '𝖆' ],
    'b' => [ '𝖇' ],
    'c' => [ '𝖈' ],
    'd' => [ '𝖉' ],
    'e' => [ '𝖊' ],
    'f' => [ '𝖋' ],
    'g' => [ '𝖌' ],
    'h' => [ '𝖍' ],
    'i' => [ '𝖎' ],
    'j' => [ '𝖏' ],
    'k' => [ '𝖐' ],
    'l' => [ '𝖑' ],
    'm' => [ '𝖒' ],
    'n' => [ '𝖓' ],
    'o' => [ '𝖔' ],
    'p' => [ '𝖕' ],
    'q' => [ '𝖖' ],
    'r' => [ '𝖗' ],
    's' => [ '𝖘' ],
    't' => [ '𝖙' ],
    'u' => [ '𝖚' ],
    'v' => [ '𝖛' ],
    'w' => [ '𝖜' ],
    'x' => [ '𝖝' ],
    'y' => [ '𝖞' ],
    'z' => [ '𝖟' ],

    'A' => [ '𝕬' ],
    'B' => [ '𝕭' ],
    'C' => [ '𝕮' ],
    'D' => [ '𝕯' ],
    'E' => [ '𝕰' ],
    'F' => [ '𝕱' ],
    'G' => [ '𝕲' ],
    'H' => [ '𝕳' ],
    'I' => [ '𝕴' ],
    'J' => [ '𝕵' ],
    'K' => [ '𝕶' ],
    'L' => [ '𝕷' ],
    'M' => [ '𝕸' ],
    'N' => [ '𝕹' ],
    'O' => [ '𝕺' ],
    'P' => [ '𝕻' ],
    'Q' => [ '𝕼' ],
    'R' => [ '𝕽' ],
    'S' => [ '𝕾' ],
    'T' => [ '𝕿' ],
    'U' => [ '𝖀' ],
    'V' => [ '𝖁' ],
    'W' => [ '𝖂' ],
    'X' => [ '𝖃' ],
    'Y' => [ '𝖄' ],
    'Z' => [ '𝖅' ],

);

our %bold-italic = (

    'a' => [ '𝒂' ],
    'b' => [ '𝒃' ],
    'c' => [ '𝒄' ],
    'd' => [ '𝒅' ],
    'e' => [ '𝒆' ],
    'f' => [ '𝒇' ],
    'g' => [ '𝒈' ],
    'h' => [ '𝒉' ],
    'i' => [ '𝒊' ],
    'j' => [ '𝒋' ],
    'k' => [ '𝒌' ],
    'l' => [ '𝒍' ],
    'm' => [ '𝒎' ],
    'n' => [ '𝒏' ],
    'o' => [ '𝒐' ],
    'p' => [ '𝒑' ],
    'q' => [ '𝒒' ],
    'r' => [ '𝒓' ],
    's' => [ '𝒔' ],
    't' => [ '𝒕' ],
    'u' => [ '𝒖' ],
    'v' => [ '𝒗' ],
    'w' => [ '𝒘' ],
    'x' => [ '𝒙' ],
    'y' => [ '𝒚' ],
    'z' => [ '𝒛' ],

    'A' => [ '𝑨' ],
    'B' => [ '𝑩' ],
    'C' => [ '𝑪' ],
    'D' => [ '𝑫' ],
    'E' => [ '𝑬' ],
    'F' => [ '𝑭' ],
    'G' => [ '𝑮' ],
    'H' => [ '𝑯' ],
    'I' => [ '𝑰' ],
    'J' => [ '𝑱' ],
    'K' => [ '𝑲' ],
    'L' => [ '𝑳' ],
    'M' => [ '𝑴' ],
    'N' => [ '𝑵' ],
    'O' => [ '𝑶' ],
    'P' => [ '𝑷' ],
    'Q' => [ '𝑸' ],
    'R' => [ '𝑹' ],
    'S' => [ '𝑺' ],
    'T' => [ '𝑻' ],
    'U' => [ '𝑼' ],
    'V' => [ '𝑽' ],
    'W' => [ '𝑾' ],
    'X' => [ '𝑿' ],
    'Y' => [ '𝒀' ],
    'Z' => [ '𝒁' ],

);

our %bold-script = (

    'a' => [ '𝓪' ],
    'b' => [ '𝓫' ],
    'c' => [ '𝓬' ],
    'd' => [ '𝓭' ],
    'e' => [ '𝓮' ],
    'f' => [ '𝓯' ],
    'g' => [ '𝓰' ],
    'h' => [ '𝓱' ],
    'i' => [ '𝓲' ],
    'j' => [ '𝓳' ],
    'k' => [ '𝓴' ],
    'l' => [ '𝓵' ],
    'm' => [ '𝓶' ],
    'n' => [ '𝓷' ],
    'o' => [ '𝓸' ],
    'p' => [ '𝓹' ],
    'q' => [ '𝓺' ],
    'r' => [ '𝓻' ],
    's' => [ '𝓼' ],
    't' => [ '𝓽' ],
    'u' => [ '𝓾' ],
    'v' => [ '𝓿' ],
    'w' => [ '𝔀' ],
    'x' => [ '𝔁' ],
    'y' => [ '𝔂' ],
    'z' => [ '𝔃' ],

    'A' => [ '𝓐' ],
    'B' => [ '𝓑' ],
    'C' => [ '𝓒' ],
    'D' => [ '𝓓' ],
    'E' => [ '𝓔' ],
    'F' => [ '𝓕' ],
    'G' => [ '𝓖' ],
    'H' => [ '𝓗' ],
    'I' => [ '𝓘' ],
    'J' => [ '𝓙' ],
    'K' => [ '𝓚' ],
    'L' => [ '𝓛' ],
    'M' => [ '𝓜' ],
    'N' => [ '𝓝' ],
    'O' => [ '𝓞' ],
    'P' => [ '𝓟' ],
    'Q' => [ '𝓠' ],
    'R' => [ '𝓡' ],
    'S' => [ '𝓢' ],
    'T' => [ '𝓣' ],
    'U' => [ '𝓤' ],
    'V' => [ '𝓥' ],
    'W' => [ '𝓦' ],
    'X' => [ '𝓧' ],
    'Y' => [ '𝓨' ],
    'Z' => [ '𝓩' ],

);

our %doublestruck = (

    '0' => [ '𝟘' ],
    '1' => [ '𝟙' ],
    '2' => [ '𝟚' ],
    '3' => [ '𝟛' ],
    '4' => [ '𝟜' ],
    '5' => [ '𝟝' ],
    '6' => [ '𝟞' ],
    '7' => [ '𝟟' ],
    '8' => [ '𝟠' ],
    '9' => [ '𝟡' ],

    'a' => [ '𝕒' ],
    'b' => [ '𝕓' ],
    'c' => [ '𝕔' ],
    'd' => [ '𝕕' ],
    'e' => [ '𝕖' ],
    'f' => [ '𝕗' ],
    'g' => [ '𝕘' ],
    'h' => [ '𝕙' ],
    'i' => [ '𝕚' ],
    'j' => [ '𝕛' ],
    'k' => [ '𝕜' ],
    'l' => [ '𝕝' ],
    'm' => [ '𝕞' ],
    'n' => [ '𝕟' ],
    'o' => [ '𝕠' ],
    'p' => [ '𝕡' ],
    'q' => [ '𝕢' ],
    'r' => [ '𝕣' ],
    's' => [ '𝕤' ],
    't' => [ '𝕥' ],
    'u' => [ '𝕦' ],
    'v' => [ '𝕧' ],
    'w' => [ '𝕨' ],
    'x' => [ '𝕩' ],
    'y' => [ '𝕪' ],
    'z' => [ '𝕫' ],

    'A' => [ '𝔸' ],
    'B' => [ '𝔹' ],
    'C' => [ 'ℂ' ],
    'D' => [ '𝔻' ],
    'E' => [ '𝔼' ],
    'F' => [ '𝔽' ],
    'G' => [ '𝔾' ],
    'H' => [ 'ℍ' ],
    'I' => [ '𝕀' ],
    'J' => [ '𝕁' ],
    'K' => [ '𝕂' ],
    'L' => [ '𝕃' ],
    'M' => [ '𝕄' ],
    'N' => [ 'ℕ' ],
    'O' => [ '𝕆' ],
    'P' => [ 'ℙ' ],
    'Q' => [ 'ℚ' ],
    'R' => [ 'ℝ' ],
    'S' => [ '𝕊' ],
    'T' => [ '𝕋' ],
    'U' => [ '𝕌' ],
    'V' => [ '𝕍' ],
    'W' => [ '𝕎' ],
    'X' => [ '𝕏' ],
    'Y' => [ '𝕐' ],
    'Z' => [ 'ℤ' ],

);

our %doublestruck-italic = (

    'd' => [ 'ⅆ' ],
    'e' => [ 'ⅇ' ],
    'i' => [ 'ⅈ' ],
    'j' => [ 'ⅉ' ],

    'D' => [ 'ⅅ' ],

);

our %fraktur = (

    'a' => [ '𝔞' ],
    'b' => [ '𝔟' ],
    'c' => [ '𝔠' ],
    'd' => [ '𝔡' ],
    'e' => [ '𝔢' ],
    'f' => [ '𝔣' ],
    'g' => [ '𝔤' ],
    'h' => [ '𝔥' ],
    'i' => [ '𝔦' ],
    'j' => [ '𝔧' ],
    'k' => [ '𝔨' ],
    'l' => [ '𝔩' ],
    'm' => [ '𝔪' ],
    'n' => [ '𝔫' ],
    'o' => [ '𝔬' ],
    'p' => [ '𝔭' ],
    'q' => [ '𝔮' ],
    'r' => [ '𝔯' ],
    's' => [ '𝔰' ],
    't' => [ '𝔱' ],
    'u' => [ '𝔲' ],
    'v' => [ '𝔳' ],
    'w' => [ '𝔴' ],
    'x' => [ '𝔵' ],
    'y' => [ '𝔶' ],
    'z' => [ '𝔷' ],

    'A' => [ '𝔄' ],
    'B' => [ '𝔅' ],
    'D' => [ '𝔇' ],
    'E' => [ '𝔈' ],
    'F' => [ '𝔉' ],
    'G' => [ '𝔊' ],
    'J' => [ '𝔍' ],
    'K' => [ '𝔎' ],
    'L' => [ '𝔏' ],
    'M' => [ '𝔐' ],
    'N' => [ '𝔑' ],
    'O' => [ '𝔒' ],
    'P' => [ '𝔓' ],
    'Q' => [ '𝔔' ],
    'S' => [ '𝔖' ],
    'T' => [ '𝔗' ],
    'U' => [ '𝔘' ],
    'V' => [ '𝔙' ],
    'W' => [ '𝔚' ],
    'X' => [ '𝔛' ],
    'Y' => [ '𝔜' ],

);

our %fullwidth = (

    '0' => [ '０' ],
    '1' => [ '１' ],
    '2' => [ '２' ],
    '3' => [ '３' ],
    '4' => [ '４' ],
    '5' => [ '５' ],
    '6' => [ '６' ],
    '7' => [ '７' ],
    '8' => [ '８' ],
    '9' => [ '９' ],

    'a' => [ 'ａ' ],
    'b' => [ 'ｂ' ],
    'c' => [ 'ｃ' ],
    'd' => [ 'ｄ' ],
    'e' => [ 'ｅ' ],
    'f' => [ 'ｆ' ],
    'g' => [ 'ｇ' ],
    'h' => [ 'ｈ' ],
    'i' => [ 'ｉ' ],
    'j' => [ 'ｊ' ],
    'k' => [ 'ｋ' ],
    'l' => [ 'ｌ' ],
    'm' => [ 'ｍ' ],
    'n' => [ 'ｎ' ],
    'o' => [ 'ｏ' ],
    'p' => [ 'ｐ' ],
    'q' => [ 'ｑ' ],
    'r' => [ 'ｒ' ],
    's' => [ 'ｓ' ],
    't' => [ 'ｔ' ],
    'u' => [ 'ｕ' ],
    'v' => [ 'ｖ' ],
    'w' => [ 'ｗ' ],
    'x' => [ 'ｘ' ],
    'y' => [ 'ｙ' ],
    'z' => [ 'ｚ' ],

    'A' => [ 'Ａ' ],
    'B' => [ 'Ｂ' ],
    'C' => [ 'Ｃ' ],
    'D' => [ 'Ｄ' ],
    'E' => [ 'Ｅ' ],
    'F' => [ 'Ｆ' ],
    'G' => [ 'Ｇ' ],
    'H' => [ 'Ｈ' ],
    'I' => [ 'Ｉ' ],
    'J' => [ 'Ｊ' ],
    'K' => [ 'Ｋ' ],
    'L' => [ 'Ｌ' ],
    'M' => [ 'Ｍ' ],
    'N' => [ 'Ｎ' ],
    'O' => [ 'Ｏ' ],
    'P' => [ 'Ｐ' ],
    'Q' => [ 'Ｑ' ],
    'R' => [ 'Ｒ' ],
    'S' => [ 'Ｓ' ],
    'T' => [ 'Ｔ' ],
    'U' => [ 'Ｕ' ],
    'V' => [ 'Ｖ' ],
    'W' => [ 'Ｗ' ],
    'X' => [ 'Ｘ' ],
    'Y' => [ 'Ｙ' ],
    'Z' => [ 'Ｚ' ],

);

our %heavy-ballot = (

    'X' => [ '✘' ],

);

our %italic = (

    'a' => [ '𝑎' ],
    'b' => [ '𝑏' ],
    'c' => [ '𝑐' ],
    'd' => [ '𝑑' ],
    'e' => [ '𝑒' ],
    'f' => [ '𝑓' ],
    'g' => [ '𝑔' ],
    'i' => [ '𝑖' ],
    'j' => [ '𝑗' ],
    'k' => [ '𝑘' ],
    'l' => [ '𝑙' ],
    'm' => [ '𝑚' ],
    'n' => [ '𝑛' ],
    'o' => [ '𝑜' ],
    'p' => [ '𝑝' ],
    'q' => [ '𝑞' ],
    'r' => [ '𝑟' ],
    's' => [ '𝑠' ],
    't' => [ '𝑡' ],
    'u' => [ '𝑢' ],
    'v' => [ '𝑣' ],
    'w' => [ '𝑤' ],
    'x' => [ '𝑥' ],
    'y' => [ '𝑦' ],
    'z' => [ '𝑧' ],

    'A' => [ '𝐴' ],
    'B' => [ '𝐵' ],
    'C' => [ '𝐶' ],
    'D' => [ '𝐷' ],
    'E' => [ '𝐸' ],
    'F' => [ '𝐹' ],
    'G' => [ '𝐺' ],
    'H' => [ '𝐻' ],
    'I' => [ '𝐼' ],
    'J' => [ '𝐽' ],
    'K' => [ '𝐾' ],
    'L' => [ '𝐿' ],
    'M' => [ '𝑀' ],
    'N' => [ '𝑁' ],
    'O' => [ '𝑂' ],
    'P' => [ '𝑃' ],
    'Q' => [ '𝑄' ],
    'R' => [ '𝑅' ],
    'S' => [ '𝑆' ],
    'T' => [ '𝑇' ],
    'U' => [ '𝑈' ],
    'V' => [ '𝑉' ],
    'W' => [ '𝑊' ],
    'X' => [ '𝑋' ],
    'Y' => [ '𝑌' ],
    'Z' => [ '𝑍' ],

);

our %monospace = (

    '0' => [ '𝟶' ],
    '1' => [ '𝟷' ],
    '2' => [ '𝟸' ],
    '3' => [ '𝟹' ],
    '4' => [ '𝟺' ],
    '5' => [ '𝟻' ],
    '6' => [ '𝟼' ],
    '7' => [ '𝟽' ],
    '8' => [ '𝟾' ],
    '9' => [ '𝟿' ],

    'a' => [ '𝚊' ],
    'b' => [ '𝚋' ],
    'c' => [ '𝚌' ],
    'd' => [ '𝚍' ],
    'e' => [ '𝚎' ],
    'f' => [ '𝚏' ],
    'g' => [ '𝚐' ],
    'h' => [ '𝚑' ],
    'i' => [ '𝚒' ],
    'j' => [ '𝚓' ],
    'k' => [ '𝚔' ],
    'l' => [ '𝚕' ],
    'm' => [ '𝚖' ],
    'n' => [ '𝚗' ],
    'o' => [ '𝚘' ],
    'p' => [ '𝚙' ],
    'q' => [ '𝚚' ],
    'r' => [ '𝚛' ],
    's' => [ '𝚜' ],
    't' => [ '𝚝' ],
    'u' => [ '𝚞' ],
    'v' => [ '𝚟' ],
    'w' => [ '𝚠' ],
    'x' => [ '𝚡' ],
    'y' => [ '𝚢' ],
    'z' => [ '𝚣' ],

    'A' => [ '𝙰' ],
    'B' => [ '𝙱' ],
    'C' => [ '𝙲' ],
    'D' => [ '𝙳' ],
    'E' => [ '𝙴' ],
    'F' => [ '𝙵' ],
    'G' => [ '𝙶' ],
    'H' => [ '𝙷' ],
    'I' => [ '𝙸' ],
    'J' => [ '𝙹' ],
    'K' => [ '𝙺' ],
    'L' => [ '𝙻' ],
    'M' => [ '𝙼' ],
    'N' => [ '𝙽' ],
    'O' => [ '𝙾' ],
    'P' => [ '𝙿' ],
    'Q' => [ '𝚀' ],
    'R' => [ '𝚁' ],
    'S' => [ '𝚂' ],
    'T' => [ '𝚃' ],
    'U' => [ '𝚄' ],
    'V' => [ '𝚅' ],
    'W' => [ '𝚆' ],
    'X' => [ '𝚇' ],
    'Y' => [ '𝚈' ],
    'Z' => [ '𝚉' ],

);

our %sansserif = (

    '0' => [ '𝟢' ],
    '1' => [ '𝟣' ],
    '2' => [ '𝟤' ],
    '3' => [ '𝟥' ],
    '4' => [ '𝟦' ],
    '5' => [ '𝟧' ],
    '6' => [ '𝟨' ],
    '7' => [ '𝟩' ],
    '8' => [ '𝟪' ],
    '9' => [ '𝟫' ],

    'a' => [ '𝖺' ],
    'b' => [ '𝖻' ],
    'c' => [ '𝖼' ],
    'd' => [ '𝖽' ],
    'e' => [ '𝖾' ],
    'f' => [ '𝖿' ],
    'g' => [ '𝗀' ],
    'h' => [ '𝗁' ],
    'i' => [ '𝗂' ],
    'j' => [ '𝗃' ],
    'k' => [ '𝗄' ],
    'l' => [ '𝗅' ],
    'm' => [ '𝗆' ],
    'n' => [ '𝗇' ],
    'o' => [ '𝗈' ],
    'p' => [ '𝗉' ],
    'q' => [ '𝗊' ],
    'r' => [ '𝗋' ],
    's' => [ '𝗌' ],
    't' => [ '𝗍' ],
    'u' => [ '𝗎' ],
    'v' => [ '𝗏' ],
    'w' => [ '𝗐' ],
    'x' => [ '𝗑' ],
    'y' => [ '𝗒' ],
    'z' => [ '𝗓' ],

    'A' => [ '𝖠' ],
    'B' => [ '𝖡' ],
    'C' => [ '𝖢' ],
    'D' => [ '𝖣' ],
    'E' => [ '𝖤' ],
    'F' => [ '𝖥' ],
    'G' => [ '𝖦' ],
    'H' => [ '𝖧' ],
    'I' => [ '𝖨' ],
    'J' => [ '𝖩' ],
    'K' => [ '𝖪' ],
    'L' => [ '𝖫' ],
    'M' => [ '𝖬' ],
    'N' => [ '𝖭' ],
    'O' => [ '𝖮' ],
    'P' => [ '𝖯' ],
    'Q' => [ '𝖰' ],
    'R' => [ '𝖱' ],
    'S' => [ '𝖲' ],
    'T' => [ '𝖳' ],
    'U' => [ '𝖴' ],
    'V' => [ '𝖵' ],
    'W' => [ '𝖶' ],
    'X' => [ '𝖷' ],
    'Y' => [ '𝖸' ],
    'Z' => [ '𝖹' ],

);

our %sansserif-bold = (

    '0' => [ '𝟬' ],
    '1' => [ '𝟭' ],
    '2' => [ '𝟮' ],
    '3' => [ '𝟯' ],
    '4' => [ '𝟰' ],
    '5' => [ '𝟱' ],
    '6' => [ '𝟲' ],
    '7' => [ '𝟳' ],
    '8' => [ '𝟴' ],
    '9' => [ '𝟵' ],

    'a' => [ '𝗮' ],
    'b' => [ '𝗯' ],
    'c' => [ '𝗰' ],
    'd' => [ '𝗱' ],
    'e' => [ '𝗲' ],
    'f' => [ '𝗳' ],
    'g' => [ '𝗴' ],
    'h' => [ '𝗵' ],
    'i' => [ '𝗶' ],
    'j' => [ '𝗷' ],
    'k' => [ '𝗸' ],
    'l' => [ '𝗹' ],
    'm' => [ '𝗺' ],
    'n' => [ '𝗻' ],
    'o' => [ '𝗼' ],
    'p' => [ '𝗽' ],
    'q' => [ '𝗾' ],
    'r' => [ '𝗿' ],
    's' => [ '𝘀' ],
    't' => [ '𝘁' ],
    'u' => [ '𝘂' ],
    'v' => [ '𝘃' ],
    'w' => [ '𝘄' ],
    'x' => [ '𝘅' ],
    'y' => [ '𝘆' ],
    'z' => [ '𝘇' ],

    'A' => [ '𝗔' ],
    'B' => [ '𝗕' ],
    'C' => [ '𝗖' ],
    'D' => [ '𝗗' ],
    'E' => [ '𝗘' ],
    'F' => [ '𝗙' ],
    'G' => [ '𝗚' ],
    'H' => [ '𝗛' ],
    'I' => [ '𝗜' ],
    'J' => [ '𝗝' ],
    'K' => [ '𝗞' ],
    'L' => [ '𝗟' ],
    'M' => [ '𝗠' ],
    'N' => [ '𝗡' ],
    'O' => [ '𝗢' ],
    'P' => [ '𝗣' ],
    'Q' => [ '𝗤' ],
    'R' => [ '𝗥' ],
    'S' => [ '𝗦' ],
    'T' => [ '𝗧' ],
    'U' => [ '𝗨' ],
    'V' => [ '𝗩' ],
    'W' => [ '𝗪' ],
    'X' => [ '𝗫' ],
    'Y' => [ '𝗬' ],
    'Z' => [ '𝗭' ],

);

our %sansserif-bold-italic = (

    'a' => [ '𝙖' ],
    'b' => [ '𝙗' ],
    'c' => [ '𝙘' ],
    'd' => [ '𝙙' ],
    'e' => [ '𝙚' ],
    'f' => [ '𝙛' ],
    'g' => [ '𝙜' ],
    'h' => [ '𝙝' ],
    'i' => [ '𝙞' ],
    'j' => [ '𝙟' ],
    'k' => [ '𝙠' ],
    'l' => [ '𝙡' ],
    'm' => [ '𝙢' ],
    'n' => [ '𝙣' ],
    'o' => [ '𝙤' ],
    'p' => [ '𝙥' ],
    'q' => [ '𝙦' ],
    'r' => [ '𝙧' ],
    's' => [ '𝙨' ],
    't' => [ '𝙩' ],
    'u' => [ '𝙪' ],
    'v' => [ '𝙫' ],
    'w' => [ '𝙬' ],
    'x' => [ '𝙭' ],
    'y' => [ '𝙮' ],
    'z' => [ '𝙯' ],

    'A' => [ '𝘼' ],
    'B' => [ '𝘽' ],
    'C' => [ '𝘾' ],
    'D' => [ '𝘿' ],
    'E' => [ '𝙀' ],
    'F' => [ '𝙁' ],
    'G' => [ '𝙂' ],
    'H' => [ '𝙃' ],
    'I' => [ '𝙄' ],
    'J' => [ '𝙅' ],
    'K' => [ '𝙆' ],
    'L' => [ '𝙇' ],
    'M' => [ '𝙈' ],
    'N' => [ '𝙉' ],
    'O' => [ '𝙊' ],
    'P' => [ '𝙋' ],
    'Q' => [ '𝙌' ],
    'R' => [ '𝙍' ],
    'S' => [ '𝙎' ],
    'T' => [ '𝙏' ],
    'U' => [ '𝙐' ],
    'V' => [ '𝙑' ],
    'W' => [ '𝙒' ],
    'X' => [ '𝙓' ],
    'Y' => [ '𝙔' ],
    'Z' => [ '𝙕' ],

);

our %sansserif-italic = (

    'a' => [ '𝘢' ],
    'b' => [ '𝘣' ],
    'c' => [ '𝘤' ],
    'd' => [ '𝘥' ],
    'e' => [ '𝘦' ],
    'f' => [ '𝘧' ],
    'g' => [ '𝘨' ],
    'h' => [ '𝘩' ],
    'i' => [ '𝘪' ],
    'j' => [ '𝘫' ],
    'k' => [ '𝘬' ],
    'l' => [ '𝘭' ],
    'm' => [ '𝘮' ],
    'n' => [ '𝘯' ],
    'o' => [ '𝘰' ],
    'p' => [ '𝘱' ],
    'q' => [ '𝘲' ],
    'r' => [ '𝘳' ],
    's' => [ '𝘴' ],
    't' => [ '𝘵' ],
    'u' => [ '𝘶' ],
    'v' => [ '𝘷' ],
    'w' => [ '𝘸' ],
    'x' => [ '𝘹' ],
    'y' => [ '𝘺' ],
    'z' => [ '𝘻' ],

    'A' => [ '𝘈' ],
    'B' => [ '𝘉' ],
    'C' => [ '𝘊' ],
    'D' => [ '𝘋' ],
    'E' => [ '𝘌' ],
    'F' => [ '𝘍' ],
    'G' => [ '𝘎' ],
    'H' => [ '𝘏' ],
    'I' => [ '𝘐' ],
    'J' => [ '𝘑' ],
    'K' => [ '𝘒' ],
    'L' => [ '𝘓' ],
    'M' => [ '𝘔' ],
    'N' => [ '𝘕' ],
    'O' => [ '𝘖' ],
    'P' => [ '𝘗' ],
    'Q' => [ '𝘘' ],
    'R' => [ '𝘙' ],
    'S' => [ '𝘚' ],
    'T' => [ '𝘛' ],
    'U' => [ '𝘜' ],
    'V' => [ '𝘝' ],
    'W' => [ '𝘞' ],
    'X' => [ '𝘟' ],
    'Y' => [ '𝘠' ],
    'Z' => [ '𝘡' ],

);

our %script = (

    'a' => [ '𝒶' ],
    'b' => [ '𝒷' ],
    'c' => [ '𝒸' ],
    'd' => [ '𝒹' ],
    'e' => [ 'ℯ' ],
    'f' => [ '𝒻' ],
    'g' => [ 'ℊ' ],
    'h' => [ '𝒽' ],
    'i' => [ '𝒾' ],
    'j' => [ '𝒿' ],
    'k' => [ '𝓀' ],
    'l' => [ 'ℓ', '𝓁' ],
    'm' => [ '𝓂' ],
    'n' => [ '𝓃' ],
    'o' => [ 'ℴ' ],
    'p' => [ '𝓅' ],
    'q' => [ '𝓆' ],
    'r' => [ '𝓇' ],
    's' => [ '𝓈' ],
    't' => [ '𝓉' ],
    'u' => [ '𝓊' ],
    'v' => [ '𝓋' ],
    'w' => [ '𝓌' ],
    'x' => [ '𝓍' ],
    'y' => [ '𝓎' ],
    'z' => [ '𝓏' ],

    'A' => [ '𝒜' ],
    'B' => [ 'ℬ' ],
    'C' => [ '𝒞' ],
    'D' => [ '𝒟' ],
    'E' => [ 'ℰ' ],
    'F' => [ 'ℱ' ],
    'G' => [ '𝒢' ],
    'H' => [ 'ℋ' ],
    'I' => [ 'ℐ' ],
    'J' => [ '𝒥' ],
    'K' => [ '𝒦' ],
    'L' => [ 'ℒ' ],
    'M' => [ 'ℳ' ],
    'N' => [ '𝒩' ],
    'O' => [ '𝒪' ],
    'P' => [ '℘', '𝒫' ],
    'Q' => [ '𝒬' ],
    'R' => [ 'ℛ' ],
    'S' => [ '𝒮' ],
    'T' => [ '𝒯' ],
    'U' => [ '𝒰' ],
    'V' => [ '𝒱' ],
    'W' => [ '𝒲' ],
    'X' => [ '𝒳' ],
    'Y' => [ '𝒴' ],
    'Z' => [ '𝒵' ],
);

our @typeface = (
    %ballot,
    %ballot-bold-script,
    %ballot-script,
    %bold,
    %bold-fraktur,
    %bold-italic,
    %bold-script,
    %doublestruck,
    %doublestruck-italic,
    %fraktur,
    %fullwidth,
    %heavy-ballot,
    %italic,
    %monospace,
    %sansserif,
    %sansserif-bold,
    %sansserif-bold-italic,
    %sansserif-italic,
    %script,
);

our %accented = (

    'a' => [ 'à', 'á', 'â', 'ã', 'ä', 'å', 'ā', 'ă', 'ą', 'ǎ', 'ǟ', 'ǡ', 'ǻ', 'ȁ', 'ȃ', 'ȧ', 'ᶏ', 'ḁ', 'ẚ', 'ạ', 'ả', 'ấ', 'ầ', 'ẩ', 'ẫ', 'ậ', 'ắ', 'ằ', 'ẳ', 'ẵ', 'ặ', 'ⱥ' ],
    'b' => [ 'ƀ', 'ƃ', 'ɓ', 'ᵬ', 'ᶀ', 'ḃ', 'ḅ', 'ḇ', 'ꞗ' ],
    'c' => [ 'ç', 'ć', 'ĉ', 'ċ', 'č', 'ƈ', 'ȼ', 'ɕ', 'ḉ', 'ꞓ', 'ꞔ' ],
    'd' => [ 'ď', 'đ', 'ƌ', 'ȡ', 'ɖ', 'ɗ', 'ᵭ', 'ᶁ', 'ᶑ', 'ḋ', 'ḍ', 'ḏ', 'ḑ', 'ḓ' ],
    'e' => [ 'è', 'é', 'ê', 'ë', 'ē', 'ĕ', 'ė', 'ę', 'ě', 'ȅ', 'ȇ', 'ȩ', 'ɇ', 'ᶒ', 'ḕ', 'ḗ', 'ḙ', 'ḛ', 'ḝ', 'ẹ', 'ẻ', 'ẽ', 'ế', 'ề', 'ể', 'ễ', 'ệ', 'ⱸ', 'ꬴ' ],
    'f' => [ 'ƒ', 'ᵮ', 'ᶂ', 'ḟ', 'ꞙ' ],
    'g' => [ 'ĝ', 'ğ', 'ġ', 'ģ', 'ǥ', 'ǧ', 'ǵ', 'ɠ', 'ᶃ', 'ḡ', 'ꞡ' ],
    'h' => [ 'ĥ', 'ħ', 'ȟ', 'ɦ', 'ḣ', 'ḥ', 'ḧ', 'ḩ', 'ḫ', 'ẖ', 'ⱨ', 'ꞕ' ],
    'i' => [ 'ì', 'í', 'î', 'ï', 'ĩ', 'ī', 'ĭ', 'į', 'ǐ', 'ȉ', 'ȋ', 'ɨ', 'ᶖ', 'ḭ', 'ḯ', 'ỉ', 'ị' ],
    'j' => [ 'ĵ', 'ǰ', 'ɉ', 'ʝ' ],
    'k' => [ 'ķ', 'ƙ', 'ǩ', 'ᶄ', 'ḱ', 'ḳ', 'ḵ', 'ⱪ', 'ꝁ', 'ꝃ', 'ꝅ', 'ꞣ' ],
    'l' => [ 'ĺ', 'ļ', 'ľ', 'ŀ', 'ł', 'ƚ', 'ȴ', 'ɫ', 'ɬ', 'ɭ', 'ᶅ', 'ḷ', 'ḹ', 'ḻ', 'ḽ', 'ⱡ', 'ꝉ', 'ꞎ', 'ꬷ', 'ꬸ', 'ꬹ' ],
    'm' => [ 'ɱ', 'ᵯ', 'ᶆ', 'ḿ', 'ṁ', 'ṃ', 'ꬺ' ],
    'n' => [ 'ñ', 'ń', 'ņ', 'ň', 'ƞ', 'ǹ', 'ȵ', 'ɲ', 'ɳ', 'ᵰ', 'ᶇ', 'ṅ', 'ṇ', 'ṉ', 'ṋ', 'ꞑ', 'ꞥ', 'ꬻ' ],
    'o' => [ 'ò', 'ó', 'ô', 'õ', 'ö', 'ø', 'ō', 'ŏ', 'ő', 'ơ', 'ǒ', 'ǫ', 'ǭ', 'ǿ', 'ȍ', 'ȏ', 'ȫ', 'ȭ', 'ȯ', 'ȱ', 'ṍ', 'ṏ', 'ṑ', 'ṓ', 'ọ', 'ỏ', 'ố', 'ồ', 'ổ', 'ỗ', 'ộ', 'ớ', 'ờ', 'ở', 'ỡ', 'ợ', 'ⱺ', 'ꝋ', 'ꝍ' ],
    'p' => [ 'ƥ', 'ᵱ', 'ᵽ', 'ᶈ', 'ṕ', 'ṗ', 'ꝑ', 'ꝓ', 'ꝕ' ],
    'q' => [ 'ɋ', 'ʠ', 'ꝗ', 'ꝙ' ],
    'r' => [ 'ŕ', 'ŗ', 'ř', 'ȑ', 'ȓ', 'ɍ', 'ɼ', 'ɽ', 'ɾ', 'ᵲ', 'ᵳ', 'ᶉ', 'ṙ', 'ṛ', 'ṝ', 'ṟ', 'ꞧ', 'ꭉ' ],
    's' => [ 'ś', 'ŝ', 'ş', 'š', 'ș', 'ȿ', 'ʂ', 'ᵴ', 'ᶊ', 'ṡ', 'ṣ', 'ṥ', 'ṧ', 'ṩ', 'ꞩ' ],
    't' => [ 'ţ', 'ť', 'ŧ', 'ƫ', 'ƭ', 'ț', 'ȶ', 'ʈ', 'ᵵ', 'ṫ', 'ṭ', 'ṯ', 'ṱ', 'ẗ', 'ⱦ' ],
    'u' => [ 'ù', 'ú', 'û', 'ü', 'ũ', 'ū', 'ŭ', 'ů', 'ű', 'ų', 'ư', 'ǔ', 'ǖ', 'ǘ', 'ǚ', 'ǜ', 'ȕ', 'ȗ', 'ᶙ', 'ṳ', 'ṵ', 'ṷ', 'ṹ', 'ṻ', 'ụ', 'ủ', 'ứ', 'ừ', 'ử', 'ữ', 'ự', 'ꞹ', 'ꭎ', 'ꭒ' ],
    'v' => [ 'ʋ', 'ᶌ', 'ṽ', 'ṿ', 'ⱱ', 'ⱴ', 'ꝟ' ],
    'w' => [ 'ŵ', 'ẁ', 'ẃ', 'ẅ', 'ẇ', 'ẉ', 'ẘ', 'ⱳ' ],
    'x' => [ 'ᶍ', 'ẋ', 'ẍ', 'ꭖ', 'ꭗ', 'ꭘ', 'ꭙ' ],
    'y' => [ 'ý', 'ÿ', 'ŷ', 'ƴ', 'ȳ', 'ɏ', 'ẏ', 'ẙ', 'ỳ', 'ỵ', 'ỷ', 'ỹ', 'ỿ', 'ꭚ' ],
    'z' => [ 'ź', 'ż', 'ž', 'ƶ', 'ȥ', 'ɀ', 'ʐ', 'ʑ', 'ᵶ', 'ᶎ', 'ẑ', 'ẓ', 'ẕ', 'ⱬ' ],

    'A' => [ 'À', 'Á', 'Â', 'Ã', 'Ä', 'Å', 'Ā', 'Ă', 'Ą', 'Ǎ', 'Ǟ', 'Ǡ', 'Ǻ', 'Ȁ', 'Ȃ', 'Ȧ', 'Ⱥ', 'Ḁ', 'Ạ', 'Ả', 'Ấ', 'Ầ', 'Ẩ', 'Ẫ', 'Ậ', 'Ắ', 'Ằ', 'Ẳ', 'Ẵ', 'Ặ' ],
    'B' => [ 'Ɓ', 'Ƃ', 'Ƀ', 'Ḃ', 'Ḅ', 'Ḇ', 'Ꞗ' ],
    'C' => [ 'Ç', 'Ć', 'Ĉ', 'Ċ', 'Č', 'Ƈ', 'Ȼ', 'Ḉ', 'Ꞓ', 'Ꞔ' ],
    'D' => [ 'Ď', 'Đ', 'Ɗ', 'Ƌ', 'Ḋ', 'Ḍ', 'Ḏ', 'Ḑ', 'Ḓ' ],
    'E' => [ 'È', 'É', 'Ê', 'Ë', 'Ē', 'Ĕ', 'Ė', 'Ę', 'Ě', 'Ȅ', 'Ȇ', 'Ȩ', 'Ɇ', 'Ḕ', 'Ḗ', 'Ḙ', 'Ḛ', 'Ḝ', 'Ẹ', 'Ẻ', 'Ẽ', 'Ế', 'Ề', 'Ể', 'Ễ', 'Ệ' ],
    'F' => [ 'Ƒ', 'Ḟ', 'Ꞙ' ],
    'G' => [ 'Ĝ', 'Ğ', 'Ġ', 'Ģ', 'Ɠ', 'Ǥ', 'Ǧ', 'Ǵ', 'Ḡ', 'Ꞡ' ],
    'H' => [ 'Ĥ', 'Ħ', 'Ȟ', 'Ḣ', 'Ḥ', 'Ḧ', 'Ḩ', 'Ḫ', 'Ⱨ', 'Ɦ' ],
    'I' => [ 'Ì', 'Í', 'Î', 'Ï', 'Ĩ', 'Ī', 'Ĭ', 'Į', 'İ', 'Ɨ', 'Ǐ', 'Ȉ', 'Ȋ', 'Ḭ', 'Ḯ', 'Ỉ', 'Ị' ],
    'J' => [ 'Ĵ', 'Ɉ', 'Ʝ' ],
    'K' => [ 'Ķ', 'Ƙ', 'Ǩ', 'Ḱ', 'Ḳ', 'Ḵ', 'Ⱪ', 'Ꝁ', 'Ꝃ', 'Ꝅ', 'Ꞣ' ],
    'L' => [ 'Ĺ', 'Ļ', 'Ľ', 'Ŀ', 'Ł', 'Ƚ', 'Ḷ', 'Ḹ', 'Ḻ', 'Ḽ', 'Ⱡ', 'Ɫ', 'Ꝉ', 'Ɬ' ],
    'M' => [ 'Ḿ', 'Ṁ', 'Ṃ', 'Ɱ' ],
    'N' => [ 'Ñ', 'Ń', 'Ņ', 'Ň', 'Ɲ', 'Ǹ', 'Ƞ', 'Ṅ', 'Ṇ', 'Ṉ', 'Ṋ', 'Ꞑ', 'Ꞥ' ],
    'O' => [ 'Ò', 'Ó', 'Ô', 'Õ', 'Ö', 'Ø', 'Ō', 'Ŏ', 'Ő', 'Ɵ', 'Ơ', 'Ǒ', 'Ǫ', 'Ǭ', 'Ǿ', 'Ȍ', 'Ȏ', 'Ȫ', 'Ȭ', 'Ȯ', 'Ȱ', 'Ṍ', 'Ṏ', 'Ṑ', 'Ṓ', 'Ọ', 'Ỏ', 'Ố', 'Ồ', 'Ổ', 'Ỗ', 'Ộ', 'Ớ', 'Ờ', 'Ở', 'Ỡ', 'Ợ', 'Ꝋ', 'Ꝍ' ],
    'P' => [ 'Ƥ', 'Ṕ', 'Ṗ', 'Ᵽ', 'Ꝑ', 'Ꝓ', 'Ꝕ' ],
    'Q' => [ 'Ꝗ', 'Ꝙ' ],
    'R' => [ 'Ŕ', 'Ŗ', 'Ř', 'Ȑ', 'Ȓ', 'Ɍ', 'Ṙ', 'Ṛ', 'Ṝ', 'Ṟ', 'Ɽ', 'Ꞧ' ],
    'S' => [ 'Ś', 'Ŝ', 'Ş', 'Š', 'Ș', 'Ṡ', 'Ṣ', 'Ṥ', 'Ṧ', 'Ṩ', 'Ȿ', 'Ꞩ', 'Ʂ' ],
    'T' => [ 'Ţ', 'Ť', 'Ŧ', 'Ƭ', 'Ʈ', 'Ț', 'Ⱦ', 'Ṫ', 'Ṭ', 'Ṯ', 'Ṱ' ],
    'U' => [ 'Ù', 'Ú', 'Û', 'Ü', 'Ũ', 'Ū', 'Ŭ', 'Ů', 'Ű', 'Ų', 'Ư', 'Ǔ', 'Ǖ', 'Ǘ', 'Ǚ', 'Ǜ', 'Ȕ', 'Ȗ', 'Ṳ', 'Ṵ', 'Ṷ', 'Ṹ', 'Ṻ', 'Ụ', 'Ủ', 'Ứ', 'Ừ', 'Ử', 'Ữ', 'Ự', 'Ꞹ' ],
    'V' => [ 'Ʋ', 'Ṽ', 'Ṿ', 'Ꝟ' ],
    'W' => [ 'Ŵ', 'Ẁ', 'Ẃ', 'Ẅ', 'Ẇ', 'Ẉ', 'Ⱳ' ],
    'X' => [ 'Ẋ', 'Ẍ' ],
    'Y' => [ 'Ý', 'Ŷ', 'Ÿ', 'Ƴ', 'Ȳ', 'Ɏ', 'Ẏ', 'Ỳ', 'Ỵ', 'Ỷ', 'Ỹ', 'Ỿ' ],
    'Z' => [ 'Ź', 'Ż', 'Ž', 'Ƶ', 'Ȥ', 'Ẑ', 'Ẓ', 'Ẕ', 'Ⱬ', 'Ɀ', 'Ᶎ' ],

);

# special mappings section

# control have very high or perfect similarity but letters are crammed
# and they are acronyms unlikely to be found in regular text
our %control = (

    'b' => [ '␢' ],

    'NUL' => [ '␀' ],
    'SOH' => [ '␁' ],
    'STX' => [ '␂' ],
    'ETX' => [ '␃' ],
    'EOT' => [ '␄' ],
    'ENQ' => [ '␅' ],
    'ACK' => [ '␆' ],
    'BEL' => [ '␇' ],
    'BS' => [ '␈' ],
    'HT' => [ '␉' ],
    'LF' => [ '␊' ],
    'VT' => [ '␋' ],
    'FF' => [ '␌' ],
    'CR' => [ '␍' ],
    'SO' => [ '␎' ],
    'SI' => [ '␏' ],
    'DLE' => [ '␐' ],
    'DC1' => [ '␑' ],
    'DC2' => [ '␒' ],
    'DC3' => [ '␓' ],
    'DC4' => [ '␔' ],
    'NAK' => [ '␕' ],
    'SYN' => [ '␖' ],
    'ETB' => [ '␗' ],
    'CAN' => [ '␘' ],
    'EM' => [ '␙' ],
    'SUB' => [ '␚' ],
    'ESC' => [ '␛' ],
    'FS' => [ '␜' ],
    'GS' => [ '␝' ],
    'RS' => [ '␞' ],
    'US' => [ '␟' ],
    'SP' => [ '␠' ],
    'DEL' => [ '␡' ],
    'NL' => [ '␤' ],

);

# flipped mappings should have very high similarity and no accents
# it may contain flips of other ASCII characters than alphanumerics
our %flipped = (

    '&' => [ '⅋' ],

    '1' => [ 'ᛚ' ],
    '3' => [ 'Ƹ' ],
    '5' => [ 'ဥ' ],
    '8' => [ '∞', 'ထ' ],

    'a' => [ 'ɐ' ],
    'c' => [ 'ɔ', 'ↄ', '𐑋', '⊃', 'ᴐ', 'ᴒ' ],
    'e' => [ 'ǝ', 'ɘ', 'ə' ],
    'g' => [ 'ᵷ' ],
    'h' => [ 'ɥ' ],
    'i' => [ 'ǃ' ],
    'k' => [ 'ʞ' ],
    'l' => [ 'ꞁ' ],
    'm' => [ 'ɯ', 'ᴟ' ],
    'o' => [ 'ᴑ' ],
    'r' => [ 'ɹ' ],
    's' => [ 'ƨ' ],
    't' => [ 'ʇ' ],
    'u' => [ 'ᴝ' ],
    'v' => [ 'ʌ', '⋀', '⋏' ],
    'w' => [ 'ʍ' ],
    'y' => [ 'ʎ' ],

    'A' => [ 'Ɐ', '∀', 'ꓯ' ],
    'B' => [ 'ꓭ' ],
    'C' => [ 'ꓛ', 'Ɔ', '𐐣' ],
    'D' => [ 'ꓷ', '⫏' ],
    'E' => [ '∃', 'ꓱ', '⧢' ],
    'F' => [ 'ꓞ' ],
    'G' => [ 'ꓨ', '⅁' ],
    'J' => [ 'Ꮣ', 'Ⴑ', '𐐓', 'ꓩ' ],
    'K' => [ 'ꓘ' ],
    'L' => [ 'Ꞁ', 'Ⴈ', '𐐑', 'ꓶ', 'Г' ],
    'N' => [ 'И', 'и', '𐐥', '𐑍' ],
    'P' => [ 'ꓒ' ],
    'Q' => [ 'Ꝺ' ],
    'R' => [ 'ᴚ', 'Я', 'ꓤ', 'я' ],
    'S' => [ 'Ƨ', 'Ƨ', 'လ' ],
    'T' => [ '⟘', 'ꓕ', '⊥', '⟂',  '⊢', '⊣' ],
    'U' => [ 'ꓵ', '∩', '⋂' ],
    'V' => [ 'Ʌ', 'ꓥ' ],
    'Y' => [ '⅄' ],

);
